//    Uploaded.to API - Java
//    Copyright (C) 2012  Julius Fischer webmaster@it-gecko.de
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package to.uploaded.file;

import java.io.IOException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.NameValuePair;
import org.apache.http.ParseException;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.message.BasicNameValuePair;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.JSONValue;

import to.uploaded.folder.UPFolder;
import to.uploaded.http.UPHttp;

public class UPFileHandler {

	private UPFileList fileList = null;
	private UPHttp upHttp;
	
	public UPFileHandler(UPHttp upHttp)
	{
		this.upHttp = upHttp;
	}
	
	/**
	 * Verschiebt eine Datei in einen Ordner
	 * @param fileId
	 * @param folderId
	 * @return
	 */
	public boolean moveToFolder(UPFile file, UPFolder folder)
	{
		return moveToFolder(file.getId(), folder.getId());
	}
	
	/**
	 * Verschiebt eine Datei in einen Ordner
	 * @param fileId
	 * @param folderId
	 * @return
	 */
	public boolean moveToFolder(String fileId, String folderId)
	{
		try {
			return this.upHttp.get("http://uploaded.to/io/me/folder/" + folderId + "/add/" + fileId).equals("");
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return false;
		
	}
	
	/**
	 * Bennent eine Datei um
	 * @param file File Objekt
	 * @param filename Neuer Dateiname mit Erweiterung! z.B. test.txt
	 * @return
	 */
	public boolean rename(UPFile file, String filename)
	{
		return rename(file.getId(), filename);
	}
	
	/**
	 * Benennt eine Datei um
	 * @param fileId File-ID
	 * @param filename Neuer Dateiname mit Erweiterung! z.B. test.txt
	 * @return
	 */
	public boolean rename(String fileId, String filename)
	{
		List<NameValuePair> formparams = new ArrayList<NameValuePair>();		
		
		try {
			
			formparams.add(new BasicNameValuePair("value", URLEncoder.encode(filename, "UTF-8")));
			formparams.add(new BasicNameValuePair("editorId", ""));
			
			return upHttp.post("http://uploaded.to/io/me/files/" + fileId + "/set/title", formparams).equals(filename);
			
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return false;
	}
	
	/**
	 * Lscht eine Datei
	 * @param file File Objekt
	 * @return
	 */
	public boolean delete(UPFile file)
	{
		return delete(file.getId());
	}
	
	/**
	 * Lscht eine Datei
	 * @param fileId File-Id
	 * @return
	 */
	public boolean delete(String fileId)
	{
		try {
			return  upHttp.get("http://uploaded.to/io/me/files/" + fileId + "/delete").equals("");
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return false;
	}
	
	/**
	 * Ld und speichert Account Files
	 * @return
	 */
	public UPFileList fetchFileList()
	{
		boolean next = false;
		Integer page = 0;
		this.fileList = new UPFileList();
		
		do
		{			
			List<NameValuePair> formparams = new ArrayList<NameValuePair>();
			
			formparams.add(new BasicNameValuePair("limit", "100"));
			formparams.add(new BasicNameValuePair("page", page.toString()));
			
			String content = "";
			
			try {
				content = upHttp.post("http://uploaded.to/io/me/list/files", formparams);
			} catch (ParseException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (ClientProtocolException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
					
			JSONObject jsonObject = (JSONObject)JSONValue.parse(content);				
			
			this.fileList.addAll(UPFileList.parseJSON((JSONArray)jsonObject.get("list")));
			
			next = ((JSONObject)jsonObject.get("listopts")).get("hasNext").equals(true);
			
			page++;
			
		}while(next);	
		
		return this.fileList;
	}
	
	/**
	 * Gibt gecachte Fileliste zurck
	 * @return
	 */
	public UPFileList getFileList()
	{
		if(fileList == null)
			fetchFileList();
		
		return fileList;
	}
}
