//    Uploaded.to API - Java
//    Copyright (C) 2012  Julius Fischer webmaster@it-gecko.de
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package to.uploaded.folder;

import java.io.IOException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.NameValuePair;
import org.apache.http.ParseException;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.message.BasicNameValuePair;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.JSONValue;

import to.uploaded.http.UPHttp;

public class UPFolderHandler {

	private UPHttp upHttp;
	private UPFolderList folderList;
	
	public UPFolderHandler(UPHttp upHttp)
	{
		this.upHttp = upHttp;
	}
	
	/**
	 * Benennt einen Ordner um
	 * @param folder Ordner-ID
	 * @param filename Ordnername
	 * @return
	 */
	public boolean rename(UPFolder folder, String filename)
	{
		return rename(folder.getId(), filename);
	}
	
	/**
	 * Benennt einen Ordner um
	 * @param folder Ordner-ID
	 * @param filename Ordnername
	 * @return
	 */
	public boolean rename(String folderId, String filename)
	{
		List<NameValuePair> formparams = new ArrayList<NameValuePair>();		
		
		try {
			
			formparams.add(new BasicNameValuePair("value", URLEncoder.encode(filename, "UTF-8")));
			formparams.add(new BasicNameValuePair("editorId", ""));
			
			return upHttp.post("http://uploaded.to/io/me/folder/" + folderId + "/set/title", formparams).equals(filename);
			
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return false;
	}
	
	/**
	 * Lscht einen Ordner
	 * @param folder Odner-ID
	 * @return
	 */
	public boolean delete(UPFolder folder)
	{
		return delete(folder.getId());
	}
	
	/**
	 * Lscht einen Ordner
	 * @param folder Odner-ID
	 * @return
	 */
	public boolean delete(String folderId)
	{
		try {
			return  upHttp.get("http://uploaded.to/io/me/folder/" + folderId + "/delete").equals("");
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return false;
	}
	
	/**
	 * Erstellt einen neunen Ordner
	 * @param name Ordnername
	 * @return
	 */
	public UPFolder create(String name)
	{		
		try {
			upHttp.get("http://uploaded.to/io/me/folder/create");
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		fetchFolderList();

		rename(folderList.findTitle("Neuer Ordner"), name);
		
		fetchFolderList();
		
		return folderList.findTitle("Neuer Ordner");
	}
	
	/**
	 * Ld eine Ordner 
	 * @return
	 */
	public UPFolderList fetchFolderList()
	{
		boolean next = false;
		Integer page = 0;
		this.folderList = new UPFolderList();
		
		do
		{			
			List<NameValuePair> formparams = new ArrayList<NameValuePair>();
			
			formparams.add(new BasicNameValuePair("page", page.toString()));
			
			String content = "";
			
			try {
				content = upHttp.post("http://uploaded.to/io/me/list/folders", formparams);
			} catch (ParseException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (ClientProtocolException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
					
			JSONObject jsonObject = (JSONObject)JSONValue.parse(content);				
			
			this.folderList.addAll(UPFolderList.parseJSON((JSONArray)jsonObject.get("list")));
			
			next = ((JSONObject)jsonObject.get("listopts")).get("hasNext").equals(true);
			
			page++;
			
		}while(next);	
		
		return this.folderList;
	}
	
	/**
	 * Gibt gecachte FolderList Objekt zurck
	 * @return
	 */
	public UPFolderList getFolderList()
	{
		if(folderList == null)
			fetchFolderList();
		
		return folderList;
	}
}
